#include <QApplication>
#include <QVBoxLayout>
#include <QPushButton>
#include <QMessageBox>
#include <QStringList>
#include <QDebug>
#include <QApplication>
#include <QThread>
#include <QMainWindow>

bool optShowDialog = true;

class FriendlyThread : public QThread
{
public:
    static void Sleep(unsigned long ms) { QThread::sleep(ms); }
};

class MainWindow : public QMainWindow
{
    Q_OBJECT
public:
    MainWindow(QWidget *parent = 0);

public slots:
    void testSlotSetCursor();
    void testSlotOverrideCursor();
};

void MainWindow::testSlotSetCursor()
{
    qDebug() << __FUNCTION__;
    // remove this line and wait cursor will show
    if (optShowDialog) {
        qDebug() << __FUNCTION__ << "##> Dialog";
        QMessageBox::information(this, tr("Information"), tr("An expensive operation will start now."), QMessageBox::Ok);
        qDebug() << __FUNCTION__ << "##< Dialog";
    }
    qDebug() << __FUNCTION__ << "##> Setting wait cursor";
    setCursor(Qt::WaitCursor);
    qDebug() << __FUNCTION__ << "##< Set wait cursor";

    FriendlyThread::Sleep(3);
    qDebug() << __FUNCTION__ << "##> Setting arrow cursor";
    setCursor(Qt::ArrowCursor);
    qDebug() << __FUNCTION__ << "##< Set arrow cursor";
    QMessageBox::information(this, tr("Information"), tr("Operation finished."), QMessageBox::Ok);
    qDebug() << __FUNCTION__ << "<";
}

void MainWindow::testSlotOverrideCursor()
{
    qDebug() << __FUNCTION__;
    // remove this line and wait cursor will show
    if (optShowDialog) {
        qDebug() << __FUNCTION__ << "##> Dialog";
        QMessageBox::information(this, tr("Information"), tr("An expensive operation will start now."), QMessageBox::Ok);
        qDebug() << __FUNCTION__ << "##< Dialog";
    }
    qDebug() << __FUNCTION__ << "##> Setting setOverrideCursor to wait cursor";
    QApplication::setOverrideCursor(Qt::WaitCursor);
    qDebug() << __FUNCTION__ << "##< Set wait cursor";

    FriendlyThread::Sleep(3);
    qDebug() << __FUNCTION__ << "##> restoreOverrideCursor";
    QApplication::restoreOverrideCursor();
    qDebug() << __FUNCTION__ << "##< restored";
    QMessageBox::information(this, tr("Information"), tr("Operation finished."), QMessageBox::Ok);
    qDebug() << __FUNCTION__ << "<";
}

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
{
    setWindowTitle(QT_VERSION_STR);
    QWidget* mainWidget = new QWidget(this);
    QVBoxLayout* layout = new QVBoxLayout(mainWidget);
    QHBoxLayout *hl =new QHBoxLayout;
    layout->addLayout(hl);
    QPushButton* button = new QPushButton(tr("open dialog (setCursor)"));
    connect(button, SIGNAL(clicked()), this, SLOT(testSlotSetCursor()));
    hl->addWidget(button);
    button = new QPushButton(tr("open dialog (Override)"));
    connect(button, SIGNAL(clicked()), this, SLOT(testSlotOverrideCursor()));
    hl->addWidget(button);
    hl->addStretch();
    layout->addStretch();

    setCentralWidget(mainWidget);
}

int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    QStringList args = QCoreApplication::arguments();
    optShowDialog = !args.contains("-d");
    qDebug() << QT_VERSION_STR;
    MainWindow w;
    w.move(200, 200);
    w.show();

    return a.exec();
}

#include "main.moc"
